#include <xsTypes.h>
#include <stlToCArrays.h>
#include <functionMap.h>
#include <XSUtil/Utils/XSutility.h>
#include <cfortran.h>
#include <cmath>

PROTOCCALLSFSUB14(DOPILE,dopile,FLOAT,INT,FLOAT,FLOAT,FLOAT,INT,INT,FLOATV,INT,FLOATV,DOUBLEV,DOUBLEV,DOUBLEV,DOUBLEV)

#define DOPILE(frame,npiled,g0,alpha,psffrac,nregions,ne,ear,ifl,photar,outar,tmpar,conv,pfrac) \
               CCALLSFSUB14(DOPILE,dopile,FLOAT,INT,FLOAT,FLOAT,FLOAT,INT,INT,FLOATV,INT,FLOATV,DOUBLEV,DOUBLEV,DOUBLEV,DOUBLEV, \
               frame,npiled,g0,alpha,psffrac,nregions,ne,ear,ifl,photar,outar,tmpar,conv,pfrac) 


void pileup(const RealArray& energyArray, const RealArray& params,
        int spectrumNumber, RealArray& flux, RealArray& fluxErr, 
        const string& initString)
{

// John Davis' forward-folding algorithm for dealing with pile-up.
// This assumes that the input photar array has been multiplied by
// the effective areas. The model parameters are the frame time (sec)
// and the number of pile-ups to consider.

// Model parameters are :
//   1     frame time
//   2     max number of piled up photons
//   3     g0 - good event fraction
//   4     alpha - grade migration factor
//   5     psffrac - fraction of the counts to pile-up
//   6     nregions - number of independent regions

// Arguments :
//    ear      r        i: Energy ranges
//    ne       i        i: Number of elements in photar array
//    param    r        i: Model parameters
//    ifl      i        i: Data set
//    photar   r      i/r: Model flux
//    photer   r      i/r: Model flux errors

// Calculates the expression

//    exp(-tau/g0 Int dE' S'(E') ) [(exp(tau F)-1)/(tau F)] o S'(E)

// where tau is the frame time, S' is the input, and F is the functional 
// defined by

//   F o f(E) = alpha Int_0^E dE' S'(E') f(E-E')

// The expression [(exp(tau F)-1)/(tau F)] is expanded in an infinite
// series

//   Sum_p  (tau F)^(p-1)/p!

   using namespace XSutility;
   
   int nE = static_cast<int>(energyArray.size()-1);
   
   float *ear=0, *pars=0, *photar=0, *photer=0;
   XSFunctions::stlToFloatArrays<float>(energyArray, params, flux, fluxErr,
        ear, pars, photar, photer);
   auto_array_ptr<float> apEar(ear);
   auto_array_ptr<float> apPars(pars);
   auto_array_ptr<float> apPhotar(photar);
   auto_array_ptr<float> apPhoter(photer);

   int nintPar2 = static_cast<int>(floor(params[1]+0.5));
   int nintPar6 = static_cast<int>(floor(params[5]+0.5));
   auto_array_ptr<double> apOutar(new double[nE]);
   auto_array_ptr<double> apTmpar(new double[nE]);
   auto_array_ptr<double> apConv(new double[nE]);
   auto_array_ptr<double> apFrac(new double[nintPar2+1]);
   double *outar = apOutar.get();
   double *tmpar = apTmpar.get();
   double *conv = apConv.get();
   double *frac = apFrac.get();
   
   DOPILE(pars[0],nintPar2,pars[2],pars[3],pars[4],nintPar6,nE,ear,spectrumNumber,
        photar,outar,tmpar,conv,frac);

   XSFunctions::floatFluxToStl<float>(photar, photer, nE, false, flux, fluxErr);
   
}
